# Build four "reflectors" at requested separations and watch them run...
# Author: Dave Greene, 1 April 2007.

from glife import getstring, validint
from time import sleep
import os
import golly as g
from glife import *

def monitor(x,y): # make sure reflection reaction is successful.
   g.setcell(x,y,1-g.getcell(x,y)) 

def main():
   answer = getstring("Enter NE/SW and NW/SE separation (N or N,M):")
   nm = answer.replace(","," ").split()
   
   if len(nm) == 0:
      n=3
      m=3
    
   # extract NE/SW and NW/SE distances
   else:
      if not validint(nm[0]):
         g.error("Bad NE/SW value: " + nm[0])
         return
      n = int(nm[0])
      if len(nm)>1:
         if not validint(nm[1]):
            g.error("Bad NW/SE value: " + nm[1])
            return
         m = int(nm[1])
      else:
         m=n
   
   all=pattern("15bo$13b3o$12bo$12b2o3$10bo$11bo$9b3o2$2b2o$bobo5b2o$bo7b2o$2o6$8b2o$8b2o!",-3,-7)\
       +pattern("2o$bo$bobo$2b2o2$11b2o$11b2o6b2o$19b2o5$10b2o$10bo$11b3o$13bo!",13+n,-12-n)\
       +pattern("6b2o$6b2o6$14b2o$5b2o7bo$5b2o5bobo$12b2o7$2b2o$3bo$3o$o!",8+n-m,-33-n-m)\
       +pattern("7bo$7b3o$10bo$9b2o5$2o$2o6b2o$8b2o2$17b2o$17bobo$19bo$19b2o!",-13-m,-23-m)
   all.display("A Wish Come True... thanks to the All-Powerful Hand of Golly")
   g.fit()
   g.setmag(g.getmag()-1) 
   g.update()
   
   reflector_monitor = dict( [ (49, [4,11]), (55, [4,11,7,11]),\
               (107+4*n, [31+n,-4-n]), (113+4*n, [31+n,-4-n,31+n,-7-n]),\
               (165+4*n+4*m, [16+n-m,-31-n-m]), (171+4*n+4*m, [16+n-m,-31-n-m,13+n-m,-31-n-m]),\
               (223+8*n+4*m, [-11-m,-16-m]), (229+8*n+4*m, [-11-m,-16-m,-11-m,-13-m]) ] )
   osc_period=232+8*n+8*m

   megamaid=pattern("""
   16b3o5b3o5b3o5b3o5b3o5b3o5b3o5b3o5b3o5b3o5b3o5b3o5b3o$15bo3bo3bo3bo3bo
   3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo$10b3ob
   o5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bob3o$9b
   o2b3o5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5b3o
   2bo$8bo3bobob3obobob3obobob3obobob3obobob3obobob3obobob3obobob3obobob
   3obobob3obobob3obobob3obobob3obobo3bo$8bo5bobobobobobobobobobobobobobo
   bobobobobobobobobobobobobobobobobobobobobobobobobobobobobobobobobobobo
   bobobo5bo$10bo3bo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bo
   bo5bobo5bo3bo$7bobo5bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo
   3bo3bo3bo3bo3bo3bo3bo3bo3bo5bobo$6b2obob2o105b2obob2o$5bobobob2o105b2o
   bobobo$4b2o3bo2bo105bo2bo3b2o$3bo3bobo2b2o103b2o2bobo3bo$6bo117bo$3b2o
   11b3o5b3o5b3o5b3o5b3o5b3o5b3o5b3o5b3o5b3o5b3o5b3o5b3o11b2o$15bo3bo3bo
   3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo$
   10b3obo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bo
   b3o$9bo2b3o5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bob
   o5b3o2bo$8bo3bobob3obobob3obobob3obobob3obobob3obobob3obobob3obobob3ob
   obob3obobob3obobob3obobob3obobob3obobo3bo$8bo5bobobobobobobobobobobobo
   bobobobobobobobobobobobobobobobobobobobobobobobobobobobobobobobobobobo
   bobobobobo5bo$10bo3bo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bob
   o5bobo5bobo5bo3bo$7bobo5bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3b
   o3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo5bobo$6b2obob2o105b2obob2o$5bobobob2o
   105b2obobobo$4b2o3bo2bo105bo2bo3b2o$3bo3bobo2b2o103b2o2bobo3bo$6bo117b
   o$3b2o11b3o5b3o5b3o5b3o5b3o5b3o5b3o5b3o5b3o5b3o5b3o5b3o5b3o11b2o$15bo
   3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo
   3bo3bo$10b3obo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo
   5bobo5bob3o$9bo2b3o5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bobo
   5bobo5bobo5b3o2bo$8bo3bobob3obobob3obobob3obobob3obobob3obobob3obobob
   3obobob3obobob3obobob3obobob3obobob3obobob3obobo3bo$8bo5bobobobobobobo
   bobobobobobobobobobobobobobobobobobobobobobobobobobobobobobobobobobobo
   bobobobobobobobobobo5bo$10bo3bo5bobo5bobo5bobo5bobo5bobo5bobo5bobo5bob
   o5bobo5bobo5bobo5bobo5bo3bo$7bobo5bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo
   3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo3bo5bobo$6b2obob2o105b2obob2o$
   5bobobob2o105b2obobobo$4b2o3bo2bo6b3ob3o78b3o11bo2bo3b2o$3bo3bobo2b2o
   4bo2bobo2bo76bo3bo9b2o2bobo3bo$6bo10bo3bobo3bo68b3o3bo5bob3o11bo$3b2o
   12bo9bo67bo3bo2bo5b3o2bo12b2o$19bo5bo62b3o3bo5bobob3obobo3bo$16bobo7bo
   bo58bo3bo2bo5bobobobobo5bo$15b2obob2ob2obob2o18b3o29b3o3bo5bobob3obobo
   5bo3bo$14bobobo2bobo2bobobo16bo3bo27bo3bo2bo5bobobobobo2bo3bo5bobo$13b
   2obo4bobo4bob2o10b3obo5bo3b3o13b3o3bo5bobob3obobo5bo9b2obob2o$12bo3bob
   2obobob2obo3bo8bo2b3o5bo2bo3bo11bo3bo2bo5bobobobobo2bo3bo10b2obobobo$
   18b3o3b3o13bo3bobob3obobo5bo3b3o3bo5bobob3obobo5bo17bo2bo3b2o$12b2ob2o
   b2o5b2ob2ob2o7bo5bobobobobo5bo2bo3bo2bo5bobobobobo2bo3bo17b2o2bobo3bo$
   11bo21bo8bo3bo5bobob3obobo5bobob3obobo5bo31bo$12bo3bo2bo5bo2bo3bo6bobo
   5bo3bo2bobobobobo5bobobobobo2bo3bo34b2o$15bo3bo5bo3bo8b2obob2o9bo5bobo
   b3obobo5bo$17bo9bo9bobobob2o10bo3bo2bobobobo2bo3bo$15bobo9bobo6b2o3bo
   2bo17bo5bo$14b2o5b3o5b2o4bo3bobo2b2o17bo3bo$16bo3bo3bo3bo9bo$19bo5bo9b
   2o17bo6b2o5b2o6bo31bo$15b2o2bo5bo2b2o23b3o5b2o5b2o5b3o29b3o$17bobob3ob
   obo18bo5b5o4bo7bo4b5o5bo15bo5b5o3bo$16bo2bobobobo2bo16b3o3b2o3b2o2b2o
   7b2o2b2o3b2o3b3o13b3o3b2o3b2ob3o$19bo5bo12bo5b5o2bo2bo2bo2bo9bo2bo2bo
   2bo2b5o5bo5b5o2bo2bo2bo3b2o$20bo3bo12b3o3b2o3b2obobobobobobo7bobobobob
   obob2o3b2o3b3o3b2o3b2obobobobobo2b2o$30bo5b5o2bo2bo2bobobobobo15bobobo
   bobo2bo2bo2b5o2bo2bo2bobobobobo4bo$29b3o3b2o3b2obobobobobo5bo15bo5bobo
   bobobob2o3b2obobobobobo5bo4b2o$22bo5b5o2bo2bo2bobobobobo31bobobobobo2b
   o2bobobobobo10bobob2o$21b3o3b2o3b2obobobobobo5bo31bo5bobobobobobo5bo9b
   2obobo$14bo5b5o2bo2bo2bobobobobo47bobobobo16bo3bobob2o$13b3o3b2o3b2obo
   bobobobo5bo47bo5bo20bo3b2o$8bo3b5o2bo2bo2bobobobobo38b3o37b2o3bob3o$7b
   3ob2o3b2obobobobobo5bo37bo3bo13bo5bo25b2o$6b2o3bo2bo2bobobobobo40b3obo
   5bo3b3o5b2o5b2o5b3o$5b2o2bobobobobobo5bo39bo2b3o5bo2bo3bo3bo9bo3bo3bo$
   6bo4bobobobo46bo3bobob3obobo5bo15bo5bo3b3o$5b2o4bo5bo46bo5bobobobobo5b
   o2b2o7b2o2bo5bo2bo3bo$3b2obobo57bo3bo5bobob3obobo11bobob3obobo5bob3o$
   4bobob2o7bo45bobo5bo3bo2bobobobo2bo9bo2bobobobobo5b3o2bo$b2obobo3bo6b
   2o5b3o35b2obob2o9bo5bo15bo5bobob3obobo3bo$b2o3bo12bo3bo3bo33bobobob2o
   10bo3bo17bo3bo2bobobobo5bo$b3obo3b2o11bo5bo3b3o25b2o3bo2bo39bo5bo3bo$
   2o16b2o2bo5bo2bo3bo23bo3bobo2b2o39bo3bo5bobo$20bobob3obobo5bo3b3o19bo
   17b3o33b2obob2o$19bo2bobobobobo5bo2bo3bo15b2o18bo3bo32b2obobobo$22bo5b
   obob3obobo5bo3b3o21b3o3bo5bo3b3o25bo2bo3b2o$23bo3bo2bobobobobo5bo2bo3b
   o19bo3bo2bo5bo2bo3bo23b2o2bobo3bo$30bo5bobob3obobo5bo3b3o5b3o3bo5bobob
   3obobo5bo3b3o23bo$31bo3bo2bobobobobo5bo2bo3bo3bo3bo2bo5bobobobobobo5bo
   2bo3bo24b2o$38bo5bobob3obobo5bobo5bobob3obobo5bobob3obobo5bo3b3o$39bo
   3bo2bobobobobo5bobo5bobobobobo2bo3bo2bobobobobo5bo2bo3bo$46bo5bobob3ob
   obob3obobo5bo9bo5bobob3obobo5bo3b3o$30bo6b2o8bo3bo2bobobobobobobobo2bo
   3bo11bo3bo2bobobobobo5bo2bo3bo$29b3o5b2o15bo5bobo5bo25bo5bobob3obobo5b
   ob3o$22bo5b5o4bo17bo3bo3bo3bo27bo3bo2bobobobobo5b3o2bo$21b3o3b2o3b2o2b
   2o64bo5bobob3obobo3bo$14bo5b5o2bo2bo2bo2bo66bo3bo2bobobobo5bo$13b3o3b
   2o3b2obobobobobobo11bo7bo7bo7bo7bo28bo5bo3bo$8bo3b5o2bo2bo2bobobobobo
   14b3o5b3o5b3o5b3o5b3o18b2o8bo3bo5bobo$7b3ob2o3b2obobobobobo5bo9bo3b5o
   3b5o3b5o3b5o3b5o3bo13b2o15b2obob2o$6b2o3bo2bo2bobobobobo16b3ob2o3b2ob
   2o3b2ob2o3b2ob2o3b2ob2o3b2ob3o12bo7b2o7b2obobobo$5b2o2bobobobobobo5bo
   15b2o3bo2bo2bobo2bo2bobo2bo2bobo2bo2bobo2bo2bo3b2o10b2o7b2o7bo2bo3b2o$
   6bo4bobobobo22b2o2bobobobobobobobobobobobobobobobobobobobobobo2b2o9bo
   8bo7b2o2bobo3bo$5b2o4bo5bo7bo15bo4bobobobobobobobobobobobobobobobobobo
   bobo4bo9bobo6b2o14bo$3b2obobo16b2o13b2o4bo5bobo5bobo5bobo5bobo5bo4b2o
   17bo17b2o$4bobob2o7bo9bo10b2obobo43bobob2o14bobo$b2obobo3bo6b2o20bobob
   2o41b2obobo$b2o3bo12bo6b2o8b2obobo3bo39bo3bobob2o$b3obo3b2o17bo7b2o3bo
   47bo3b2o$2o16b2o7bo8b3obo3b2o39b2o3bob3o$20bo14b2o57b2o$19bo2$34bo3bo
   51bo3bo$33b3ob3o49b3ob3o$32b2obobob2o47b2obobob2o$31b2o7b2o45b2o7b2o$
   32bo7bo47bo7bo$31b2o7b2o45b2o7b2o$29b2obobo3bobob2o16bo7bo16b2obobo3bo
   bob2o$30bobob2ob2obobo16b3o5b3o16bobob2ob2obobo$27b2obo2bobobobo2bob2o
   6bo5b5o3b5o5bo6b2obo2bobobobo2bob2o$27b2obob2obobob2obob2o5b3o3b2o3b2o
   b2o3b2o3b3o5b2obob2obobob2obob2o$27bobo2bo2bobo2bo2bobo4b5o2bo2bo2bobo
   2bo2bo2b5o4bobo2bo2bobo2bo2bobo$26b2ob2o11b2ob2o2b2o3b2obobobobobobobo
   bob2o3b2o2b2ob2o11b2ob2o$26bo4b2obo3bob2o4bo2bo2bo2bobobobobobobobobob
   o2bo2bo2bo4b2obo3bob2o4bo$25bobob5o5b5obobobobobobobo5bobo5bobobobobob
   obob5o5b5obobo$49bobobobo17bobobobo$30bobo7bobo6bo5bo17bo5bo6bobo7bobo
   $32bo7bo47bo7bo$28b2o6bo6b2o14b3o5b3o14b2o6bo6b2o$28b2o5b3o5b2o13bo3bo
   3bo3bo13b2o5b3o5b2o$29bo4b5o4bo9b3obo5bobo5bob3o9bo4b5o4bo$29b2o2b2o3b
   2o2b2o8bo2b3o5bobo5b3o2bo8b2o2b2o3b2o2b2o$30bo2bo2bo2bo2bo8bo3bobob3ob
   obob3obobo3bo8bo2bo2bo2bo2bo$29bobobobobobobobo7bo5bobobobobobobobo5bo
   7bobobobobobobobo$33bobobobo13bo3bo5bobo5bo3bo13bobobobo$33bo5bo10bobo
   5bo3bo3bo3bo5bobo10bo5bo$32bobo3bobo8b2obob2o17b2obob2o8bobo3bobo$32bo
   2bobo2bo7bobobob2o17b2obobobo7bo2bobo2bo$35bobo9b2o3bo2bo17bo2bo3b2o9b
   obo$33bo5bo6bo3bobo2b2o15b2o2bobo3bo6bo5bo$34b2ob2o10bo29bo10b2ob2o$
   46b2o33b2o!""")
      
   wbfn=pattern("""
   o3bob4o3b3o2b3o3b2o2bo2bob4o$o3bobo6bo2bobo2bobo2bobobo2bo$obobob3o4b
   3o2b3o2b4ob2o3b3o$2ob2obo6bo2bobo2bobo2bobobo2bo$o3bob4o3b3o2bo2bobo2b
   obo2bob4o3$3o2bo2b2o3bo2bo2bo2b2o3bo2b2o2bobo$o3bobobobo2bo2bobobobobo
   bobobobobobo$2o2bobob2o3b2obobobob2o2bobobobobobo$o3bobobobo2bob2obobo
   bobobobobobo2bo$o4bo2bobo2bo2bo2bo2b2o3bo2b2o3bo!""")
   y=401 + osc_period + n*4 + m*4
   wbfn.put(-8,y/2)
   megamaid.put(-55,y/2-102)
   
   run_flag=False
   try:
      while True:
         ch = g.getkey()
         if ch==" ":
            run_flag=False
         if ch==chr(13):
            run_flag=not run_flag
         elif ch==" " or run_flag==True:
            g.select([-8,y/2,37,12])
            g.clear(0)
            g.select([-105,y/2-105,230,2])
            g.clear(0)
            g.select([])
            y=y-1
            g.run(1)
            wbfn.put(-8,y/2)
            monitor_flag=False # avoid triggering continuously when stopped
         else:
            g.dokey(ch) # just pass keys through and let Golly handle them
         # if run_flag==True:
            # sleep(0) # TODO: work in effects for speed keys here
         gen=int(g.getgen()) % osc_period
         if gen in reflector_monitor and monitor_flag==False:
            monitor_flag=True
            keycells=reflector_monitor[gen]
            for i in range(0,len(keycells),2):
               # disable monitoring if necessary after ship arrives
               if keycells[i+1]<y/2-120:
                  monitor(keycells[i], keycells[i+1]) 
         g.update()
   except:
      g.new("Untitled") # hide incriminating evidence
      g.show("") # clear the script message before exiting

# -----------------------------------------------------

main()
